/* ------------------------------------------------------------------------
 * Copyright 2013 Tim Vernum
 * ------------------------------------------------------------------------
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ------------------------------------------------------------------------
 */
package org.adjective.syntactic.convert.j8to7.type;

import org.adjective.syntactic.parser.ast.ASTClassOrInterfaceBodyElement;
import org.adjective.syntactic.parser.ast.ASTClassOrInterfaceDeclaration;
import org.adjective.syntactic.parser.ast.ASTMember;
import org.adjective.syntactic.parser.ast.ASTMethodDeclaration;
import org.adjective.syntactic.parser.ast.ASTTypeParameter;
import org.adjective.syntactic.parser.name.TypeParameter;
import org.adjective.syntactic.parser.type.JavaType;
import org.adjective.syntactic.parser.type.MemberType;
import org.adjective.syntactic.parser.util.ModifierSet;

import java.util.ArrayList;
import java.util.List;

public class ASTTypeInfo extends AbstractTypeInfo implements TypeInfo, TypeParameterLookup
{
    private final ASTClassOrInterfaceDeclaration _declaration;
    private final TypeParameter[] _parameters;
    private final ASTMethodInfo[] _methods;

    public ASTTypeInfo(final ASTClassOrInterfaceDeclaration declaration, final TypeParameter[] parameters)
    {
        _declaration = declaration;
        _parameters = parameters;
        _methods = buildMethods();
    }

    private ASTMethodInfo[] buildMethods()
    {
        final List<ASTMethodInfo> result = new ArrayList<ASTMethodInfo>();
        for (ASTClassOrInterfaceBodyElement element : _declaration.getBody().getElements())
        {
            if (!element.isMember())
            {
                continue;
            }
            final ASTMember member = element.getMember();
            if (member.getMember().getMemberType() != MemberType.METHOD)
            {
                continue;
            }
            ModifierSet modifiers = getModifiers(member);
            result.add(new ASTMethodInfo(modifiers, (ASTMethodDeclaration) member.getMember(), this));
        }
        return result.toArray(new ASTMethodInfo[result.size()]);
    }

    private ModifierSet getModifiers(final ASTMember member)
    {
        ModifierSet modifiers = member.getModifiers();
        if (this._declaration.isInterface())
        {
            modifiers = new ModifierSet(modifiers);
            modifiers.set(ModifierSet.Modifier.ABSTRACT);
            modifiers.set(ModifierSet.Modifier.PUBLIC);
        }
        return modifiers;
    }

    @Override
    public MethodInfo[] getMethods()
    {
        return _methods;
    }

    @Override
    public String getName()
    {
        return _declaration.getName().getIdentifier();
    }

    @Override
    public JavaType findParameter(final String name)
    {
        Integer index = findParameterIndex(name);
        if (index == null)
        {
            return null;
        }
        else
        {
            return _parameters[index].getType();
        }
    }

    @Override
    public Integer findParameterIndex(final String name)
    {
        int i = 0;
        for (ASTTypeParameter parameter : _declaration.getTypeParameters().getParameters())
        {
            if (parameter.getIdentifier().getIdentifier().equals(name))
            {
                return i;
            }
            i++;
        }
        return null;
    }

}
