/* ------------------------------------------------------------------------
 * Copyright 2013 Tim Vernum
 * ------------------------------------------------------------------------
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ------------------------------------------------------------------------
 */
package org.adjective.syntactic.convert.j8to7.type;

import org.adjective.syntactic.util.JavaTypeUtil;

import java.lang.reflect.Method;

public abstract class AbstractMethodInfo implements  MethodInfo {

    protected boolean isDeclaredOnObject()
    {
        final Method[] objectMethods = Object.class.getMethods();
        for (Method objectMethod : objectMethods)
        {
            if (matches(objectMethod))
            {
                return true;
            }
        }
        return false;
    }

    private boolean matches(final Method objectMethod)
    {
        if (!objectMethod.getName().equals(getName()))
        {
            return false;
        }
        if (objectMethod.getParameterTypes().length != getParameterTypes().length)
        {
            return false;
        }
        for (int i = 0; i < objectMethod.getParameterTypes().length; i++)
        {
            if (!JavaTypeUtil.isMatchingType(getParameterTypes()[i], objectMethod.getParameterTypes()[i]))
            {
                return false;
            }
        }
        return true;
    }

    @Override
    public int getParameterCount()
    {
        return getParameterTypes().length;
    }
}
