/*
 * ------------------------------------------------------------------------
 *  Copyright 2013 Tim Vernum
 *  ------------------------------------------------------------------------
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *  ------------------------------------------------------------------------
 */
package org.adjective.syntactic.convert.j8to7.type;

import org.adjective.syntactic.parser.name.TypeParameter;
import org.adjective.syntactic.parser.type.JavaType;

import java.lang.reflect.TypeVariable;

public class ClassInfo extends AbstractTypeInfo implements TypeInfo, TypeParameterLookup
{
    private final Class<?> _cls;
    private final TypeParameter[] _parameters;

    public ClassInfo(final Class<?> cls, final TypeParameter[] parameters)
    {
        _cls = cls;
        _parameters = parameters;
    }

    @Override
    public MethodInfo[] getMethods()
    {
        return ClassMethodInfo.array(_cls.getMethods(), this);
    }

    @Override
    public String getName()
    {
        return _cls.getName();
    }

    @Override
    public JavaType findParameter(String name)
    {
        final Integer index = findParameterIndex(name);
        return index == null ? null : _parameters[index].getType();
    }

    @Override
    public Integer findParameterIndex(final String name)
    {
        final TypeVariable<? extends Class<?>>[] typeParameters = _cls.getTypeParameters();
        for (int i = 0; i < typeParameters.length; i++)
        {
            TypeVariable<? extends Class<?>> typeParameter = typeParameters[i];
            if (typeParameter.getName().equals(name))
            {
                return i;
            }
        }
        return null;
    }

    @Override
    public String toString()
    {
        if (_parameters.length == 0)
        {
            return _cls.getName();
        }
        StringBuilder builder = new StringBuilder(_cls.getName());
        builder.append("<");
        for (TypeParameter parameter : _parameters)
        {
            builder.append(parameter).append(",");
        }
        builder.setCharAt(builder.length() - 1, '>');
        return builder.toString();
    }
}
