/* ------------------------------------------------------------------------
 * Copyright 2013 Tim Vernum
 * ------------------------------------------------------------------------
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ------------------------------------------------------------------------
 */
package org.adjective.syntactic.convert.j8to7.type;

import org.adjective.syntactic.parser.type.BaseType;
import org.adjective.syntactic.parser.type.JavaType;
import org.adjective.syntactic.parser.name.ParameterizedName;
import org.adjective.syntactic.parser.name.UnparameterizedName;

public class ClassType implements JavaType
{
    private Class<?> _cls;

    public ClassType(Class<?> cls)
    {
        _cls = cls;
    }

    @Override
    public BaseType getBaseType()
    {
        final Class<?> cls = getBaseComponentClass();
        if (cls.isPrimitive())
        {
            if (cls == Void.TYPE)
            {
                return BaseType.VOID;
            }
            if (cls == Boolean.TYPE)
            {
                return BaseType.BOOLEAN;
            }
            if (cls == Byte.TYPE)
            {
                return BaseType.BYTE;
            }
            if (cls == Short.TYPE)
            {
                return BaseType.SHORT;
            }
            if (cls == Integer.TYPE)
            {
                return BaseType.INT;
            }
            if (cls == Long.TYPE)
            {
                return BaseType.LONG;
            }
            if (cls == Float.TYPE)
            {
                return BaseType.FLOAT;
            }
            if (cls == Double.TYPE)
            {
                return BaseType.DOUBLE;
            }
            if (cls == Character.TYPE)
            {
                return BaseType.CHAR;
            }
        }
        return BaseType.OBJECT;
    }

    private Class<?> getBaseComponentClass()
    {
        Class<?> cls = _cls;
        while(cls.isArray()) {
            cls = cls.getComponentType();
        }
        return cls;
    }

    @Override
    public String getTypeName()
    {
        return getBaseComponentClass().getName();
    }

    @Override
    public ParameterizedName[] getParameterizedTypeName()
    {
        final String[] parts = getTypeName().split("\\.");
        final ParameterizedName[] names = new ParameterizedName[parts.length];
        for (int i = 0; i < names.length; i++)
        {
            names[i] = new UnparameterizedName(parts[i]);
        }
        return names;
    }

    @Override
    public int getArrayDepth()
    {
        int depth = 0;
        for (Class cls = _cls; cls.isArray(); cls = cls.getComponentType())
        {
            depth++;
        }
        return depth;
    }

    public static ClassType[] array(final Class<?>... cls)
    {
        ClassType[] type = new ClassType[cls.length];
        for (int i = 0; i < cls.length; i++)
        {
            type[i] = new ClassType(cls[i]);
        }
        return type;
    }
}
