/*
 * ------------------------------------------------------------------------
 *  Copyright 2013 Tim Vernum
 *  ------------------------------------------------------------------------
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *  ------------------------------------------------------------------------
 */
package org.adjective.syntactic.convert.j8to7.type;

import org.adjective.syntactic.parser.type.JavaType;
import org.adjective.syntactic.parser.util.ModifierSet;

public interface MethodInfo {

    public String getName();
    public ModifierSet getModifiers();
    public JavaType getReturnType();
    public int getParameterCount();
    public JavaType[] getParameterTypes();
    public String[] getParameterNames();

    /**
     * @return An array of integers representing, for each of the parameters of this method, the zero-based index of the enclosing class's type parameters are represented by that method parameter (or null if the method parameter is not a class type parameter)
     * e.g. For the declaration
     * <pre>interface C&lt;S,T&gt; {
     *   void f( String s, T t );
     * }</pre>
     * This method would return <code>{ null, 1 }</code>
     */
    public Integer[] getClassTypeParameters();
    public boolean isVarArgs();
    public JavaType[] getExceptionTypes();
}
