/*
 * ------------------------------------------------------------------------
 *  Copyright 2013 Tim Vernum
 *  ------------------------------------------------------------------------
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *  ------------------------------------------------------------------------
 */
package org.adjective.syntactic.convert.j8to7.util;

import org.adjective.syntactic.parser.ast.ASTImportDeclaration;
import org.adjective.syntactic.parser.ast.ASTQualifiedIdentifier;
import org.adjective.syntactic.parser.type.JavaType;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

public class Importer
{
    private final ClassFinder _classFinder;
    private final Map<String, String> typeImports;
    private final List<String> typeOnDemandImports;

    public Importer(final ClassFinder classFinder)
    {
        _classFinder = classFinder;
        this.typeImports = new LinkedHashMap<String, String>();
        this.typeOnDemandImports = new ArrayList<String>();
    }

    public void add(final ASTImportDeclaration node)
    {
        if (node.isStatic())
        {
            return;
        }
        final ASTQualifiedIdentifier identifier = node.getIdentifier();
        if (node.isOnDemand())
        {
            typeOnDemandImports.add(identifier.getName());
        }
        else
        {
            typeImports.put(identifier.getLastIdentifier().getIdentifier(), identifier.getName());
        }
    }

    public String getQualifiedName(String simpleName)
    {
        final String importedName = typeImports.get(simpleName);
        if (importedName != null)
        {
            return importedName;
        }
        for (String pkg : typeOnDemandImports)
        {
            final String qualifiedName = lookupClass(pkg, simpleName);
            if (qualifiedName != null)
            {
                return qualifiedName;
            }
        }
        return lookupClass("java.lang", simpleName);
    }

    private String lookupClass(final String pkg, final String simpleName)
    {
        final String qualifiedName = pkg + "." + simpleName;
        if (classExists(qualifiedName))
        {
            return qualifiedName;
        }
        else
        {
            return null;
        }
    }

    private boolean classExists(final String qualifiedName)
    {
        return _classFinder.hasClass(qualifiedName);
    }

    public String getQualifiedName(final JavaType type)
    {
        String name = type.getTypeName();
        if (type.getParameterizedTypeName().length == 1)
        {
            String qN = getQualifiedName(name);
            if (qN != null)
            {
                name = qN;
            }
        }
        return name;
    }
}
