/*
 * ------------------------------------------------------------------------
 *  Copyright 2013 Tim Vernum
 *  ------------------------------------------------------------------------
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *  ------------------------------------------------------------------------
 */
package org.adjective.syntactic.convert.j8to7.util;

import org.adjective.syntactic.parser.node.VariableDeclarationNode;

import java.util.HashMap;
import java.util.Map;
import java.util.Stack;

public class VariableStack
{
    private Stack<Map<String, VariableDeclarationNode>> _stack;

    public VariableStack()
    {
        _stack = new Stack<Map<String, VariableDeclarationNode>>();
    }

    public void pushScope()
    {
        _stack.push(new HashMap<String, VariableDeclarationNode>());
    }

    public void popScope()
    {
        _stack.pop();
    }

    public void store(String name, VariableDeclarationNode var)
    {
        final Map<String, VariableDeclarationNode> map = _stack.peek();
        if (map.containsKey(name))
        {
            throw new IllegalArgumentException("Variable '" + name + "' already exists in current scope");
        }
        map.put(name, var);
    }

    public VariableDeclarationNode get(String name)
    {
        for (int i = _stack.size() - 1; i >= 0; i--)
        {
            final VariableDeclarationNode var = _stack.get(i).get(name);
            if (var != null)
            {
                return var;
            }
        }
        return null;
    }

}
