package org.adjective.syntactic.parser.ast;

import org.adjective.syntactic.parser.node.BaseNode;
import org.adjective.syntactic.parser.jj.JavaParserConstants;
import org.adjective.syntactic.parser.jj.Token;
import org.adjective.syntactic.parser.node.ArrayDimensionsAndInitialization;
import org.adjective.syntactic.parser.node.ExpressionNode;
import org.adjective.syntactic.parser.type.JavaType;
import org.adjective.syntactic.parser.node.TypeNode;

public class ASTAllocationExpression extends BaseNode implements ExpressionNode
{
    public ASTAllocationExpression(int id)
    {
        super(id);
    }

    public ASTAllocationExpression(final JavaType type, final ASTArguments arguments, final ASTClassOrInterfaceBody body)
    {
        this(JavaParserTreeConstants.JJTALLOCATIONEXPRESSION);
        final Token tokenNew = new Token();
        tokenNew.image = "new";
        tokenNew.kind= JavaParserConstants.NEW;
        jjtSetFirstToken(tokenNew);
        setChildren(super.makeTypeNode(type), arguments);
        if (body != null)
        {
            appendChild(body);
        }
    }

    public <R, T> R jjtAccept(JavaParserVisitor<R, T> visitor, T data)
    {
        return visitor.visit(this, data);
    }

    public boolean isArray()
    {
        return super.getLastChild().is(ArrayDimensionsAndInitialization.class);
    }

    public boolean isAnonymousClass()
    {
        return getClassBody() != null;
    }

    public ASTClassOrInterfaceBody getClassBody()
    {
        return optionalChild(ASTClassOrInterfaceBody.class);
    }

    public boolean hasTypeArguments()
    {
        return getTypeArguments() != null;
    }

    public TypeNode getAllocationType()
    {
        return getFirstChild().as(TypeNode.class);
    }

    public ASTTypeArguments getTypeArguments()
    {
        return optionalChild(ASTTypeArguments.class);
    }

    public ASTArguments getConstructorArguments()
    {
        return optionalChild(ASTArguments.class);
    }

    public ArrayDimensionsAndInitialization getArrayNode()
    {
        return getLastChild().as(ArrayDimensionsAndInitialization.class);
    }
}
