/*
 * ------------------------------------------------------------------------
 *  Copyright 2013 Tim Vernum
 *  ------------------------------------------------------------------------
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *  ------------------------------------------------------------------------
 */
package org.adjective.syntactic.parser.ast;

import org.adjective.syntactic.parser.node.BaseNode;
import org.adjective.syntactic.parser.node.IsMember;
import org.adjective.syntactic.parser.type.MemberType;
import org.adjective.syntactic.parser.node.TypeNode;

public class ASTAnnotationMethodDeclaration extends BaseNode implements IsMember
{
    public ASTAnnotationMethodDeclaration(int id)
    {
        super(id);
    }

    @Override
    public <R, T> R jjtAccept(final JavaParserVisitor<R, T> visitor, final T data)
    {
        return visitor.visit(this, data);
    }

    public MemberType getMemberType()
    {
        return MemberType.METHOD;
    }

    public TypeNode getType()
    {
        return getFirstChild().as(TypeNode.class);
    }

    public ASTIdentifier getName()
    {
        return child(1).as(ASTIdentifier.class);
    }

    public ASTDefaultValue getDefaultValue()
    {
        return optionalChild(ASTDefaultValue.class);
    }
}
