package org.adjective.syntactic.parser.ast;

import org.adjective.syntactic.parser.node.BaseNode;
import org.adjective.syntactic.parser.node.ExpressionSuffix;

import java.util.Iterator;

public class ASTArguments extends BaseNode implements ExpressionSuffix
{
    public ASTArguments(int id)
    {
        super(id);
    }

    public ASTArguments()
    {
        this(JavaParserTreeConstants.JJTARGUMENTS);
    }

    public ASTArguments(ASTArgumentList argumentList)
    {
        this();
        setChildren(argumentList);
    }

    public <R, T> R jjtAccept(JavaParserVisitor<R, T> visitor, T data)
    {
        return visitor.visit(this, data);
    }

    public ASTArgumentList getArguments()
    {
        return optionalChild(ASTArgumentList.class);
    }

    public ASTExpression firstArgument()
    {
        return getArguments().getArguments().first();
    }

    public ASTArguments shift()
    {
        final ASTArgumentList arguments = getArguments();
        final Iterator<ASTExpression> iterator = arguments.getArguments().iterator();
        iterator.next();

        final ASTExpression[] expressions = new ASTExpression[arguments.jjtGetNumChildren() - 1];
        for (int i = 0; i < expressions.length; i++)
        {
            expressions[i] = iterator.next();
        }

        return new ASTArguments(new ASTArgumentList(expressions));
    }

    public int getNumberOfArguments()
    {
        final ASTArgumentList arguments = getArguments();
        if (arguments == null)
        {
            return 0;
        }
        return arguments.jjtGetNumChildren();
    }
}
