package org.adjective.syntactic.parser.ast;

import org.adjective.syntactic.parser.node.BaseNode;
import org.adjective.syntactic.parser.jj.JavaParserConstants;
import org.adjective.syntactic.parser.jj.Token;

public class ASTAssignmentOperator extends BaseNode
{
    public enum AssignmentOperator
    {
        ASSIGN, MULTIPLY_ASSIGN, DIVIDE_ASSIGN, MODULUS_ASSIGN, ADD_ASSIGN, SUBTRACT_ASSIGN,
        LEFT_SHIFT_ASSIGN, RIGHT_SIGNED_SHIFT_ASSIGN, RIGHT_UNSIGNED_SHIFT_ASSIGN, AND_ASSIGN,
        XOR_ASSIGN, OR_ASSIGN
    }

    public ASTAssignmentOperator(int id)
    {
        super(id);
    }

    public <R, T> R jjtAccept(JavaParserVisitor<R, T> visitor, T data)
    {
        return visitor.visit(this, data);
    }

    public AssignmentOperator getOperator()
    {
        final Token token = jjtGetFirstToken();
        switch (token.kind)
        {
            case JavaParserConstants.ASSIGN:
                return AssignmentOperator.ASSIGN;
            case JavaParserConstants.STARASSIGN:
                return AssignmentOperator.MULTIPLY_ASSIGN;
            case JavaParserConstants.SLASHASSIGN:
                return AssignmentOperator.DIVIDE_ASSIGN;
            case JavaParserConstants.REMASSIGN:
                return AssignmentOperator.MODULUS_ASSIGN;
            case JavaParserConstants.PLUSASSIGN:
                return AssignmentOperator.ADD_ASSIGN;
            case JavaParserConstants.MINUSASSIGN:
                return AssignmentOperator.SUBTRACT_ASSIGN;
            case JavaParserConstants.LSHIFTASSIGN:
                return AssignmentOperator.LEFT_SHIFT_ASSIGN;
            case JavaParserConstants.RSIGNEDSHIFTASSIGN:
                return AssignmentOperator.RIGHT_SIGNED_SHIFT_ASSIGN;
            case JavaParserConstants.RUNSIGNEDSHIFTASSIGN:
                return AssignmentOperator.RIGHT_UNSIGNED_SHIFT_ASSIGN;
            case JavaParserConstants.ANDASSIGN:
                return AssignmentOperator.AND_ASSIGN;
            case JavaParserConstants.XORASSIGN:
                return AssignmentOperator.XOR_ASSIGN;
            case JavaParserConstants.ORASSIGN:
                return AssignmentOperator.OR_ASSIGN;
        }
        throw new IllegalStateException("PostfixOperator token " + token.kind
                                                + " (" + JavaParserConstants.tokenImage[token.kind] + " ) "
                                                + token.image + " not understood");
    }
}
