package org.adjective.syntactic.parser.ast;

import org.adjective.syntactic.parser.node.BaseNode;
import org.adjective.syntactic.parser.jj.JavaParser;
import org.adjective.syntactic.parser.jj.JavaParserConstants;
import org.adjective.syntactic.parser.jj.Token;

public class ASTBinaryOperator extends BaseNode
{
    public enum BinaryOperator
    {
        EQUAL, NOT_EQUAL, GREATER_THAN, LESS_THAN, GREATER_EQUAL, LESS_EQUAL,
        PLUS, MINUS, MULTIPLY, DIVIDE, MODULUS, LEFT_SHIFT, RIGHT_SIGNED_SHIFT, RIGHT_UNSIGNED_SHIFT
    }

    public ASTBinaryOperator(int id)
    {
        super(id);
    }

    public <R, T> R jjtAccept(JavaParserVisitor<R, T> visitor, T data)
    {
        return visitor.visit(this, data);
    }

    public BinaryOperator getOperator()
    {
        final Token token = jjtGetFirstToken();
        switch (token.kind)
        {
            case JavaParser.EQ:
                return BinaryOperator.EQUAL;
            case JavaParser.NE:
                return BinaryOperator.NOT_EQUAL;
            case JavaParser.GT:
                return BinaryOperator.GREATER_THAN;
            case JavaParser.LT:
                return BinaryOperator.LESS_THAN;
            case JavaParser.GE:
                return BinaryOperator.GREATER_EQUAL;
            case JavaParser.LE:
                return BinaryOperator.LESS_EQUAL;
            case JavaParser.PLUS:
                return BinaryOperator.PLUS;
            case JavaParser.MINUS:
                return BinaryOperator.MINUS;
            case JavaParser.STAR:
                return BinaryOperator.MULTIPLY;
            case JavaParser.SLASH:
                return BinaryOperator.DIVIDE;
            case JavaParser.REM:
                return BinaryOperator.MODULUS;
            case JavaParser.LSHIFT:
                return BinaryOperator.LEFT_SHIFT;
            case JavaParser.RSIGNEDSHIFT:
                return BinaryOperator.RIGHT_SIGNED_SHIFT;
            case JavaParser.RUNSIGNEDSHIFT:
                return BinaryOperator.RIGHT_UNSIGNED_SHIFT;
        }
        throw new IllegalStateException("Unexpected binary operator (" +
                                                token.kind + ":" + JavaParserConstants.tokenImage[token.kind] +
                                                ")" + token.image);
    }
}
