package org.adjective.syntactic.parser.ast;

import org.adjective.syntactic.parser.node.BaseNode;
import org.adjective.syntactic.parser.node.ExpressionNode;

public class ASTCharacterLiteral extends BaseNode implements ExpressionNode
{
    public ASTCharacterLiteral(int id)
    {
        super(id);
    }

    public <R, T> R jjtAccept(JavaParserVisitor<R, T> visitor, T data)
    {
        return visitor.visit(this, data);
    }

    public String getImage()
    {
        return jjtGetFirstToken().image;
    }

    public char getCharacter()
    {
        String image = jjtGetFirstToken().image;
        if (image.length() == 3)
        {
            // simple literal
            return image.charAt(1);
        }
        final char ch = image.charAt(2);
        switch (ch)
        {
            case '\\':
            case '\'':
            case '"':
                return ch;
            case 'n':
                return '\n';
            case 't':
                return '\t';
            case 'b':
                return '\b';
            case 'r':
                return '\r';
            case 'f':
                return '\f';
        }
        if (Character.isDigit(ch))
        {
            String oct = image.substring(2, image.length() - 1);
            int val = Integer.parseInt(oct, 8);
            return (char) val;
        }
        throw new IllegalStateException("Unsupported character literal: " + image);
    }
}
