package org.adjective.syntactic.parser.ast;

import org.adjective.syntactic.parser.node.BaseNode;
import org.adjective.syntactic.parser.type.DeclarationType;
import org.adjective.syntactic.parser.node.IsTypeDeclaration;
import org.adjective.syntactic.parser.type.MemberType;
import org.adjective.syntactic.parser.node.StatementNode;

public class ASTClassOrInterfaceDeclaration extends BaseNode implements IsTypeDeclaration, StatementNode
{
    public ASTClassOrInterfaceDeclaration(int id)
    {
        super(id);
    }

    public <R,T> R jjtAccept(JavaParserVisitor<R,T> visitor, T data)
    {
        return visitor.visit(this, data);
    }

    @Override
    public DeclarationType getDeclarationType()
    {
        return isInterface() ? DeclarationType.INTERFACE : DeclarationType.CLASS;
    }

    @Override
    public MemberType getMemberType()
    {
        return isInterface() ? MemberType.INTERFACE : MemberType.CLASS;
    }

    public boolean isInterface()
    {
        return child(ASTClassOrInterfaceKeyword.class).isInterface();
    }

    public ASTIdentifier getName()
    {
        return child(ASTIdentifier.class);
    }

    public ASTTypeParameters getTypeParameters()
    {
        return optionalChild(ASTTypeParameters.class);
    }

    public ASTExtendsList getExtends()
    {
        return optionalChild(ASTExtendsList.class);
    }

    public ASTImplementsList getImplements()
    {
        return optionalChild(ASTImplementsList.class);
    }

    public ASTClassOrInterfaceBody getBody()
    {
        return child(ASTClassOrInterfaceBody.class);
    }
}
