package org.adjective.syntactic.parser.ast;

import org.adjective.syntactic.parser.node.BaseNode;
import org.adjective.syntactic.parser.type.BaseType;
import org.adjective.syntactic.parser.name.ParameterizedName;
import org.adjective.syntactic.parser.node.TypeNode;

import java.util.ArrayList;
import java.util.List;

public class ASTClassOrInterfaceType extends BaseNode implements TypeNode
{
    public ASTClassOrInterfaceType(int id)
    {
        super(id);
    }

    public ASTClassOrInterfaceType(final ASTParameterizedIdentifier[] nameComponents)
    {
        this(JavaParserTreeConstants.JJTCLASSORINTERFACETYPE);
        setChildren(nameComponents);
    }

    public ASTClassOrInterfaceType(final ParameterizedName[] components)
    {
        this(JavaParserTreeConstants.JJTCLASSORINTERFACETYPE);
        ASTParameterizedIdentifier[] children = new ASTParameterizedIdentifier[components.length];
        for (int i = 0; i < components.length; i++)
        {
            if (components[i] instanceof ASTParameterizedIdentifier)
            {
                children[i] = (ASTParameterizedIdentifier) components[i];
            }
            else
            {
                children[i] = new ASTParameterizedIdentifier(components[i]);
            }
        }
        setChildren(children);
    }

    public <R, T> R jjtAccept(JavaParserVisitor<R, T> visitor, T data)
    {
        return visitor.visit(this, data);
    }

    @Override
    public BaseType getBaseType()
    {
        return BaseType.OBJECT;
    }

    @Override
    public String getTypeName()
    {
        List<ASTIdentifier> identifiers = new ArrayList<ASTIdentifier>();
        for (ASTParameterizedIdentifier param : getComponents())
        {
            identifiers.add(param.getIdentifier());
        }
        return getIdentifierName(identifiers);
    }

    @Override
    public ParameterizedName[] getParameterizedTypeName()
    {
        final ParameterizedName[] names = new ParameterizedName[jjtGetNumChildren()];
        int i = 0;
        for (ASTParameterizedIdentifier identifier : getComponents())
        {
            names[i++] = identifier;
        }
        return names;
    }

    public Iterable<ASTParameterizedIdentifier> getComponents()
    {
        return allChildren(ASTParameterizedIdentifier.class);
    }

    @Override
    public int getArrayDepth()
    {
        return 0;
    }
}
