package org.adjective.syntactic.parser.ast;

import org.adjective.syntactic.parser.node.BaseNode;
import org.adjective.syntactic.parser.node.ExpressionNode;

public class ASTExpression extends BaseNode implements ExpressionNode
{
    public ASTExpression(int id)
    {
        super(id);
    }

    public ASTExpression(final ExpressionNode expression)
    {
        this(JavaParserTreeConstants.JJTEXPRESSION);
        setChildren(expression);
    }

    public <R,T> R jjtAccept(JavaParserVisitor<R,T> visitor, T data)
    {
        return visitor.visit(this, data);
    }

    public ExpressionNode getBaseExpression()
    {
        return getFirstChild().as(ExpressionNode.class);
    }

    public boolean isAssignment()
    {
        return jjtGetNumChildren() > 1;
    }

    public ASTAssignmentOperator getOperator()
    {
        return optionalChild(ASTAssignmentOperator.class);
    }

    public ASTExpression getAssignmentRHS()
    {
        return jjtGetNumChildren() == 3 ? child(2).as(ASTExpression.class) : null;
    }

    public static ASTExpression[] array(final ExpressionNode[] expressions)
    {
        final ASTExpression[] ast = new ASTExpression[expressions.length];
        for (int i = 0; i < ast.length; i++)
        {
            ast[i] = new ASTExpression(expressions[i]);
        }
        return ast;
    }
}
