/* ------------------------------------------------------------------------
 * Copyright 2012 Tim Vernum
 * ------------------------------------------------------------------------
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ------------------------------------------------------------------------
 */
package org.adjective.syntactic.parser.ast;

import org.adjective.syntactic.parser.node.BaseNode;
import org.adjective.syntactic.parser.node.HasAnnotations;
import org.adjective.syntactic.parser.node.HasModifiers;
import org.adjective.syntactic.parser.util.ModifierSet;
import org.adjective.syntactic.parser.node.TypeNode;

public class ASTForEach extends BaseNode implements ASTForStatement.ForCondition, HasAnnotations, HasModifiers
{
    public ASTForEach(final int i)
    {
        super(i);
    }

    @Override
    public <R,T> R jjtAccept(final JavaParserVisitor<R,T> visitor, final T data)
    {
        return visitor.visit(this, data);
    }

    public ASTModifiers getModifiersNode()
    {
        return getFirstChild().as(ASTModifiers.class);
    }

    @Override
    public Iterable<ASTAnnotation> getAnnotations()
    {
        return getModifiersNode().getAnnotations();
    }

    @Override
    public ModifierSet getModifiers()
    {
        return getModifiersNode().getModifiers();
    }

    public TypeNode getTypeNode()
    {
        return child(1).as(TypeNode.class);
    }

    public ASTIdentifier getIdentifier()
    {
        return child(2).as(ASTIdentifier.class);
    }

    public ASTExpression getIterable()
    {
        return child(3).as(ASTExpression.class);
    }
}
