package org.adjective.syntactic.parser.ast;

import org.adjective.syntactic.parser.node.BaseNode;
import org.adjective.syntactic.parser.node.LambdaParameter;
import org.adjective.syntactic.parser.type.JavaType;
import org.adjective.syntactic.parser.util.ModifierSet;
import org.adjective.syntactic.parser.node.TypeNode;
import org.adjective.syntactic.parser.node.VariableDeclarationNode;

import java.util.Collections;
import java.util.Map;

public class ASTFormalParameter extends BaseNode implements VariableDeclarationNode, LambdaParameter
{
    public ASTFormalParameter(int id)
    {
        super(id);
    }

    public ASTFormalParameter(final JavaType type, boolean varargs, String name)
    {
        this(JavaParserTreeConstants.JJTFORMALPARAMETER);
        setChildren(new ASTVariableModifiers(),
                    makeTypeNode(type));
        if (varargs)
        {
            appendChild(new ASTVarArgs());
        }
        appendChild(new ASTIdentifier(name));
        appendChild(new ASTArraySuffixList());
    }

    public <R, T> R jjtAccept(JavaParserVisitor<R, T> visitor, T data)
    {
        return visitor.visit(this, data);
    }

    @Override
    public Iterable<ASTAnnotation> getAnnotations()
    {
        return child(ASTVariableModifiers.class).getAnnotations();
    }

    public ModifierSet getModifiers()
    {
        return child(ASTVariableModifiers.class).getModifiers();
    }

    public TypeNode getTypeNode()
    {
        return child(TypeNode.class);
    }

    public boolean isVarArgs()
    {
        return optionalChild(ASTVarArgs.class) != null;
    }

    public ASTIdentifier getName()
    {
        return child(ASTIdentifier.class);
    }

    public ASTArraySuffixList getArraySuffix()
    {
        return child(ASTArraySuffixList.class);
    }

    public JavaType getActualType()
    {
        return BaseNode.makeType(getTypeNode(), getArraySuffix());
    }

    @Override
    public boolean isMultiVariable()
    {
        return false;
    }

    @Override
    public Map<String, VariableDeclarationNode> split()
    {
        return Collections.<String, VariableDeclarationNode>singletonMap(getName().getIdentifier(), this);
    }
}
