package org.adjective.syntactic.parser.ast;

import org.adjective.syntactic.parser.node.BaseNode;
import org.adjective.syntactic.parser.type.JavaType;

public class ASTFormalParameters extends BaseNode
{
    public ASTFormalParameters(int id)
    {
        super(id);
    }

    public ASTFormalParameters(ASTFormalParameter... formalParameters)
    {
        this(JavaParserTreeConstants.JJTFORMALPARAMETERS);
        setChildren(formalParameters);
    }

    public <R, T> R jjtAccept(JavaParserVisitor<R, T> visitor, T data)
    {
        return visitor.visit(this, data);
    }

    public Iterable<ASTFormalParameter> getParameters()
    {
        return allChildren(ASTFormalParameter.class);
    }

    public JavaType[] getParameterTypes()
    {
        final JavaType[] types = new JavaType[jjtGetNumChildren()];
        int i = 0;
        for (ASTFormalParameter parameter : getParameters())
        {
            types[i++] = parameter.getActualType();
        }
        return types;
    }

    public String[] getParameterNames()
    {
        final String[] names = new String[jjtGetNumChildren()];
        int i = 0;
        for (ASTFormalParameter parameter : getParameters())
        {
            names[i++] = parameter.getName().getIdentifier();
        }
        return names;
    }

    public boolean isVarArgs()
    {
        return !isEmpty() && getLastChild().as(ASTFormalParameter.class).isVarArgs();
    }
}
