package org.adjective.syntactic.parser.ast;

import org.adjective.syntactic.parser.node.BaseNode;
import org.adjective.syntactic.parser.jj.JavaParser;
import org.adjective.syntactic.parser.jj.JavaParserConstants;
import org.adjective.syntactic.parser.jj.Token;
import org.adjective.syntactic.parser.node.ExpressionNode;

import java.math.BigInteger;

public class ASTIntegerLiteral extends BaseNode implements ExpressionNode
{
    public ASTIntegerLiteral(int id)
    {
        super(id);
    }

    public <R, T> R jjtAccept(JavaParserVisitor<R, T> visitor, T data)
    {
        return visitor.visit(this, data);
    }

    public static class IntegerValue
    {
        public final long value;
        public final boolean isLong;

        public IntegerValue(long val, boolean lng)
        {
            value = val;
            isLong = lng;
        }

        @Override
        public String toString()
        {
            final String str = Long.toString(value);
            if (isLong)
            {
                return str + "L";
            }
            else
            {
                return str;
            }
        }
    }

    public IntegerValue getValue()
    {
        final Token token = jjtGetFirstToken();
        boolean isLong = false;
        String image = token.image;
        if (image.endsWith("l") || image.endsWith("L"))
        {
            isLong = true;
            image = image.substring(0, image.length() - 1);
        }
        image = image.replace("_", "");
        final int kind = token.kind;
        return new IntegerValue(parse(image, kind, isLong), isLong);
    }

    private long parse(final String image, final int kind, final boolean isLong)
    {
        switch (kind)
        {
            case JavaParser.DECIMAL_LITERAL:
                return Long.parseLong(image, 10);
            case JavaParser.HEX_LITERAL:
                return parseHex(image.substring(2), isLong);
            case JavaParser.OCTAL_LITERAL:
                return parseOctal(image.substring(1), isLong);
            case JavaParser.BINARY_LITERAL:
                return parseBinary(image.substring(2), isLong);
        }
        throw new IllegalArgumentException("Unknown token type " + JavaParserConstants.tokenImage[kind]);
    }

    private long parseBinary(final String image, final boolean isLong)
    {
        // TODO Clone hex code here
        if (isLong)
        {
            if (image.equals("1111111111111111111111111111111111111111111111111111111111111111"))
            {
                return -1;
            }
        }
        else
        {
            if (image.equals("11111111111111111111111111111111"))
            {
                return -1;
            }
        }
        return Long.parseLong(image, 2);
    }

    private long parseOctal(final String image, final boolean isLong)
    {
        // TODO Clone hex code here
        if (isLong)
        {
            if (image.equals("1000000000000000000000"))
            {
                return -1;
            }
        }
        else
        {
            if (image.equals("3777777777"))
            {
                return -1;
            }
        }
        return Long.parseLong(image, 8);
    }

    private long parseHex(final String image, final boolean isLong)
    {
        if (isLong)
        {
            if (image.toLowerCase().matches("[8-9a-f]..............."))
            {
                final BigInteger dec = new BigInteger(image, 16);
                return dec.subtract(new BigInteger("10000000000000000", 16)).longValue();
            }
        }
        else
        {
            if (image.toLowerCase().matches("[8-9a-f]......."))
            {
                final BigInteger dec = new BigInteger(image, 16);
                return dec.subtract(new BigInteger("100000000", 16)).longValue();
            }
        }
        return Long.parseLong(image, 16);
    }
}
