package org.adjective.syntactic.parser.ast;

import org.adjective.syntactic.parser.node.BaseNode;
import org.adjective.syntactic.parser.node.LambdaParameter;
import org.adjective.syntactic.parser.type.JavaType;

import java.util.ArrayList;

public class ASTLambdaParameterList extends BaseNode
{
    public ASTLambdaParameterList(int id)
    {
        super(id);
    }

    public <R, T> R jjtAccept(JavaParserVisitor<R, T> visitor, T data)
    {
        return visitor.visit(this, data);
    }

    public Iterable<LambdaParameter> getParameters()
    {
        return allChildren(LambdaParameter.class);
    }

    public Iterable<ASTIdentifier> getParameterNames()
    {
        if (hasImplicitTypes())
        {
            return allChildren(ASTIdentifier.class);
        }
        final ArrayList<ASTIdentifier> identifiers = new ArrayList<ASTIdentifier>();
        for (ASTFormalParameter param : allChildren(ASTFormalParameter.class))
        {
            identifiers.add(param.getName());
        }
        return identifiers;
    }

    public boolean hasDeclaredTypes()
    {
        return getFirstChild().is(ASTFormalParameter.class);
    }

    public boolean hasImplicitTypes()
    {
        return getFirstChild().is(ASTIdentifier.class);
    }

    public JavaType[] getParameterTypes()
    {
        if (hasImplicitTypes())
        {
            return null;
        }
        JavaType[] types = new JavaType[jjtGetNumChildren()];
        int i = 0;
        for (ASTFormalParameter param : allChildren(ASTFormalParameter.class))
        {
            types[i++] = param.getActualType();
        }
        return types;
    }
}
