package org.adjective.syntactic.parser.ast;

import org.adjective.syntactic.parser.node.BaseNode;

import java.util.Collections;

public class ASTLambdaParameters extends BaseNode
{
    public ASTLambdaParameters(int id)
    {
        super(id);
    }

    @Override
    public <R,T> R jjtAccept(JavaParserVisitor<R,T> visitor, T data)
    {
        return visitor.visit(this, data);
    }

    public boolean isSingleParameter()
    {
        return !isEmpty() && getFirstChild().is(ASTIdentifier.class);
    }

    public Iterable<ASTIdentifier> getParameterNames()
    {
        if(isEmpty()) {
            return Collections.emptyList();
        }
        if (isSingleParameter())
        {
            return Collections.singleton(getSingleParameterNode());
        }
        else
        {
            return getParameterListNode().getParameterNames();
        }
    }

    public ASTIdentifier getSingleParameterNode()
    {
        return getFirstChild().as(ASTIdentifier.class);
    }

    public ASTLambdaParameterList getParameterListNode()
    {
        return getFirstChild().as(ASTLambdaParameterList.class);
    }

    public boolean hasDeclaredParameterTypes()
    {
        return !isEmpty() && !isSingleParameter() && getParameterListNode().hasDeclaredTypes();
    }
}
