package org.adjective.syntactic.parser.ast;

import org.adjective.syntactic.parser.node.BaseNode;
import org.adjective.syntactic.parser.node.ForLoopInit;
import org.adjective.syntactic.parser.util.ModifierSet;
import org.adjective.syntactic.parser.node.TypeNode;
import org.adjective.syntactic.parser.node.VariableDeclarationNode;

import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.Map;

public class ASTLocalVariableDeclaration extends BaseNode implements VariableDeclarationNode, ForLoopInit
{
    public ASTLocalVariableDeclaration(int id)
    {
        super(id);
    }

    public ASTLocalVariableDeclaration(final ASTModifiers modifiersNode, final TypeNode typeNode, final ASTVariableDeclarator... declarators)
    {
        this(JavaParserTreeConstants.JJTLOCALVARIABLEDECLARATION);
        appendChild(modifiersNode);
        appendChild(typeNode);
        appendChildren(declarators);
    }

    public <R, T> R jjtAccept(JavaParserVisitor<R, T> visitor, T data)
    {
        return visitor.visit(this, data);
    }

    public ModifierSet getModifiers()
    {
        return getModifiersNode().getModifiers();
    }

    public Iterable<ASTAnnotation> getAnnotations()
    {
        return getModifiersNode().getAnnotations();
    }

    public ASTModifiers getModifiersNode()
    {
        return getFirstChild().as(ASTModifiers.class);
    }

    public TypeNode getTypeNode()
    {
        return child(TypeNode.class);
    }

    public Iterable<ASTVariableDeclarator> getVariables()
    {
        return findChildren(ASTVariableDeclarator.class);
    }

    @Override
    public boolean isMultiVariable()
    {
        return countChildren(ASTVariableDeclarator.class) > 1;
    }

    @Override
    public Map<String, VariableDeclarationNode> split()
    {
        if (!isMultiVariable())
        {
            final ASTVariableDeclarator declarator = getVariables().iterator().next();
            final String var = getName(declarator);
            return Collections.<String, VariableDeclarationNode>singletonMap(var, this);
        }
        final Map<String, VariableDeclarationNode> map = new LinkedHashMap<String, VariableDeclarationNode>();
        for (ASTVariableDeclarator declarator : getVariables())
        {
            map.put(getName(declarator), cloneWith(declarator));
        }
        return map;
    }

    private ASTLocalVariableDeclaration cloneWith(final ASTVariableDeclarator declarator)
    {
        return new ASTLocalVariableDeclaration(new ASTModifiers(getModifiersNode()), getTypeNode(), declarator);
    }

    private String getName(final ASTVariableDeclarator declarator)
    {
        return declarator.getIdentifier().getIdentifier();
    }
}
