/* ------------------------------------------------------------------------
 * Copyright 2012 Tim Vernum
 * ------------------------------------------------------------------------
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ------------------------------------------------------------------------
 */
package org.adjective.syntactic.parser.ast;

import org.adjective.syntactic.parser.node.BaseNode;
import org.adjective.syntactic.parser.node.HasAnnotations;
import org.adjective.syntactic.parser.node.HasModifiers;
import org.adjective.syntactic.parser.node.IsMember;
import org.adjective.syntactic.parser.util.ModifierSet;

public class ASTMember extends BaseNode implements HasAnnotations, HasModifiers
{
    public ASTMember(int i)
    {
        super(i);
    }

    public ASTMember(final ASTModifiers modifiers, final IsMember member)
    {
        this(JavaParserTreeConstants.JJTMEMBER);
        jjtAddChild(member, 1);
        jjtAddChild(modifiers, 0);
    }

    public <R, T> R jjtAccept(JavaParserVisitor<R, T> visitor, T data)
    {
        return visitor.visit(this, data);
    }

    @Override
    public Iterable<ASTAnnotation> getAnnotations()
    {
        return getModifiersNode().getAnnotations();
    }

    public IsMember getMember()
    {
        return child(1).as(IsMember.class);
    }

    @Override
    public ModifierSet getModifiers()
    {
        return getModifiersNode().getModifiers();
    }

    private ASTModifiers getModifiersNode()
    {
        return child(ASTModifiers.class);
    }
}
