package org.adjective.syntactic.parser.ast;

import org.adjective.syntactic.parser.node.BaseNode;
import org.adjective.syntactic.parser.node.IsMember;
import org.adjective.syntactic.parser.type.JavaType;
import org.adjective.syntactic.parser.type.MemberType;
import org.adjective.syntactic.parser.node.TypeNode;

public class ASTMethodDeclaration extends BaseNode implements IsMember
{
    public ASTMethodDeclaration(int id)
    {
        super(id);
    }

    public ASTMethodDeclaration(final JavaType type, final ASTIdentifier name, final ASTFormalParameters parameters, final ASTArraySuffixList suffixList, final ASTNameList exceptions, final ASTMethodBody body)
    {
        this(JavaParserTreeConstants.JJTMETHODDECLARATION);
        setChildren(makeTypeNode(type), name, parameters, suffixList);
        if (exceptions != null)
        {
            appendChild(exceptions);
        }
        appendChild(body);
    }

    public <R, T> R jjtAccept(JavaParserVisitor<R, T> visitor, T data)
    {
        return visitor.visit(this, data);
    }

    @Override
    public MemberType getMemberType()
    {
        return MemberType.METHOD;
    }

    public ASTTypeParameters getTypeParameters()
    {
        return optionalChild(ASTTypeParameters.class);
    }

    public TypeNode getDeclaredReturnType()
    {
        return child(TypeNode.class);
    }

    public ASTIdentifier getName()
    {
        return child(ASTIdentifier.class);
    }

    public ASTFormalParameters getParameters()
    {
        return child(ASTFormalParameters.class);
    }

    public ASTArraySuffixList getArraySuffix()
    {
        return child(ASTArraySuffixList.class);
    }

    public ASTNameList getThrows()
    {
        return optionalChild(ASTNameList.class);
    }

    public ASTMethodBody getBody()
    {
        return child(ASTMethodBody.class);
    }

    public JavaType getActualReturnType()
    {
        return makeType(getDeclaredReturnType(), getArraySuffix());
    }
}
