package org.adjective.syntactic.parser.ast;

import org.adjective.syntactic.parser.node.BaseNode;
import org.adjective.syntactic.parser.node.HasAnnotations;
import org.adjective.syntactic.parser.util.ModifierSet;

public class ASTModifiers extends BaseNode implements HasAnnotations
{
    private ModifierSet _modifiers;

    public ASTModifiers(ModifierSet modifiers)
    {
        this(JavaParserTreeConstants.JJTMODIFIERS);
        _modifiers = modifiers;
    }

    public ASTModifiers(int id)
    {
        super(id);
        _modifiers = null;
    }

    public ASTModifiers(final ASTModifiers clone)
    {
        this(new ModifierSet(clone.getModifiers()));
        setChildren(clone.children());
    }

    public <R, T> R jjtAccept(JavaParserVisitor<R, T> visitor, T data)
    {
        return visitor.visit(this, data);
    }

    @Override
    public Iterable<ASTAnnotation> getAnnotations()
    {
        return allChildren(ASTAnnotation.class);
    }

    public void setModifiers(ModifierSet modifiers)
    {
        if (_modifiers == null)
        {
            _modifiers = new ModifierSet(modifiers);
        }
        else
        {
            throw new IllegalStateException("Modifiers have already been set");
        }
    }

    public ModifierSet getModifiers()
    {
        return _modifiers;
    }
}
