package org.adjective.syntactic.parser.ast;

import org.adjective.syntactic.parser.node.BaseNode;
import org.adjective.syntactic.parser.type.JavaType;
import org.adjective.syntactic.parser.name.ParameterizedName;
import org.adjective.syntactic.parser.type.SimpleJavaType;

public class ASTNameList extends BaseNode
{
    public ASTNameList(int id)
    {
        super(id);
    }

    public ASTNameList(final JavaType... exceptionType)
    {
        this(JavaParserTreeConstants.JJTNAMELIST);
        ASTQualifiedIdentifier[] identifiers = new ASTQualifiedIdentifier[exceptionType.length];
        for (int i = 0; i < identifiers.length; i++)
        {
            final String[] names = getNameComponents(exceptionType[i]);
            identifiers[i] = new ASTQualifiedIdentifier(names);
        }
        setChildren(identifiers);
    }

    private String[] getNameComponents(final JavaType javaType)
    {
        final ParameterizedName[] parameterizedNames = javaType.getParameterizedTypeName();
        final String[] names = new String[parameterizedNames.length];
        for (int j = 0; j < names.length; j++)
        {
            names[j] = parameterizedNames[j].getName();
        }
        return names;
    }

    public <R, T> R jjtAccept(JavaParserVisitor<R, T> visitor, T data)
    {
        return visitor.visit(this, data);
    }

    public Iterable<ASTQualifiedIdentifier> getNames()
    {
        return allChildren(ASTQualifiedIdentifier.class);
    }

    public JavaType[] getTypes()
    {
        final JavaType[] types = new JavaType[jjtGetNumChildren()];
        int i = 0;
        for (ASTQualifiedIdentifier name : getNames())
        {
            types[i++] = new SimpleJavaType(name);
        }
        return types;
    }
}
