package org.adjective.syntactic.parser.ast;

import org.adjective.syntactic.parser.node.BaseNode;
import org.adjective.syntactic.parser.name.ParameterizedName;
import org.adjective.syntactic.parser.name.TypeParameter;

public class ASTParameterizedIdentifier extends BaseNode implements  ParameterizedName
{
    public ASTParameterizedIdentifier(final int i)
    {
        super(i);
    }

    public ASTParameterizedIdentifier(final ParameterizedName name)
    {
        this(JavaParserTreeConstants.JJTPARAMETERIZEDIDENTIFIER);
        appendChild(new ASTIdentifier(name.getName()));
        if (name.getParameters().length != 0)
        {
            appendChild(new ASTTypeArguments(name.getParameters()));
        }
    }

    @Override
    public <R, T> R jjtAccept(final JavaParserVisitor<R, T> visitor, final T data)
    {
        return visitor.visit(this, data);
    }

    public ASTIdentifier getIdentifier()
    {
        return getFirstChild().as(ASTIdentifier.class);
    }

    public ASTTypeArguments getTypeArguments()
    {
        return optionalChild(ASTTypeArguments.class);
    }

    @Override
    public TypeParameter[] getParameters()
    {
        final ASTTypeArguments arguments = getTypeArguments();
        return arguments == null ? new TypeParameter[0] : arguments.getParameters();
    }

    @Override
    public String getName()
    {
        return getIdentifier().getIdentifier();
    }
}
