package org.adjective.syntactic.parser.ast;

import org.adjective.syntactic.parser.node.BaseNode;
import org.adjective.syntactic.parser.jj.JavaParserConstants;
import org.adjective.syntactic.parser.jj.Token;
import org.adjective.syntactic.parser.name.ParameterizedName;
import org.adjective.syntactic.parser.name.UnparameterizedName;
import org.adjective.syntactic.parser.node.TypeNode;
import org.adjective.syntactic.parser.type.BaseType;

public class ASTPrimitiveType extends BaseNode implements TypeNode
{
    public ASTPrimitiveType(int id)
    {
        super(id);
    }

    public ASTPrimitiveType(final BaseType baseType)
    {
        this(JavaParserTreeConstants.JJTPRIMITIVETYPE);
        final Token token = new Token();
        switch (baseType)
        {
            case BOOLEAN:
                token.image = "boolean";
                token.kind = JavaParserConstants.BOOLEAN;
                break;
            case BYTE:
                token.image = "byte";
                token.kind = JavaParserConstants.BYTE;
                break;
            case CHAR:
                token.image = "char";
                token.kind = JavaParserConstants.CHAR;
                break;
            case DOUBLE:
                token.image = "double";
                token.kind = JavaParserConstants.DOUBLE;
                break;
            case FLOAT:
                token.image = "float";
                token.kind = JavaParserConstants.FLOAT;
                break;
            case INT:
                token.image = "int";
                token.kind = JavaParserConstants.INT;
                break;
            case LONG:
                token.image = "long";
                token.kind = JavaParserConstants.LONG;
                break;
            case SHORT:
                token.image = "short";
                token.kind = JavaParserConstants.SHORT;
                break;
            default:
                throw new IllegalArgumentException("Not a primitive type: " + baseType);
        }
        jjtSetFirstToken(token);
    }

    public <R, T> R jjtAccept(JavaParserVisitor<R, T> visitor, T data)
    {
        return visitor.visit(this, data);
    }

    @Override
    public BaseType getBaseType()
    {
        final int kind = jjtGetFirstToken().kind;
        switch (kind)
        {
            case JavaParserConstants.BOOLEAN:
                return BaseType.BOOLEAN;
            case JavaParserConstants.BYTE:
                return BaseType.BYTE;
            case JavaParserConstants.CHAR:
                return BaseType.CHAR;
            case JavaParserConstants.DOUBLE:
                return BaseType.DOUBLE;
            case JavaParserConstants.FLOAT:
                return BaseType.FLOAT;
            case JavaParserConstants.INT:
                return BaseType.INT;
            case JavaParserConstants.LONG:
                return BaseType.LONG;
            case JavaParserConstants.SHORT:
                return BaseType.SHORT;
        }
        throw new IllegalStateException("Invalid primitive token type: " + kind
                                                + " (" + JavaParserConstants.tokenImage[kind] + ")");
    }

    @Override
    public String getTypeName()
    {
        return jjtGetFirstToken().image;
    }

    @Override
    public ParameterizedName[] getParameterizedTypeName()
    {
        return new ParameterizedName[] {
                new UnparameterizedName(getTypeName())
        };
    }

    @Override
    public int getArrayDepth()
    {
        return 0;
    }

}
