/* ------------------------------------------------------------------------
 * Copyright 2012 Tim Vernum
 * ------------------------------------------------------------------------
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ------------------------------------------------------------------------
 */
package org.adjective.syntactic.parser.ast;

import org.adjective.syntactic.parser.node.BaseNode;
import org.adjective.syntactic.parser.type.BaseType;
import org.adjective.syntactic.parser.type.JavaType;
import org.adjective.syntactic.parser.name.ParameterizedName;
import org.adjective.syntactic.parser.node.TypeNode;

public class ASTReferenceType extends BaseNode implements TypeNode
{
    public ASTReferenceType(int i)
    {
        super(i);
    }

    public ASTReferenceType(final TypeNode componentType, final int arrayDepth)
    {
        this(JavaParserTreeConstants.JJTREFERENCETYPE);
        jjtAddChild(componentType, 0);
        for(int i=1;i<=arrayDepth; i++){
            jjtAddChild(new ASTArraySuffix(), i);
        }
    }

    @Override
    public <R,T> R jjtAccept(JavaParserVisitor<R,T> visitor, T data)
    {
        return visitor.visit(this, data);
    }

    public TypeNode getComponentType()
    {
        return getFirstChild().as(TypeNode.class);
    }

    @Override
    public int getArrayDepth()
    {
        return super.countChildren(ASTArraySuffix.class);
    }

    @Override
    public BaseType getBaseType()
    {
        return getComponentType().getBaseType();
    }

    @Override
    public String getTypeName()
    {
        return getComponentType().getTypeName();
    }

    @Override
    public ParameterizedName[] getParameterizedTypeName()
    {
        return getComponentType().getParameterizedTypeName();
    }

    public JavaType getActualType()
    {
        return makeArrayType(getComponentType(), getArrayDepth());
    }
}
