/*
 * ------------------------------------------------------------------------
 *  Copyright 2013 Tim Vernum
 *  ------------------------------------------------------------------------
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *  ------------------------------------------------------------------------
 */

package org.adjective.syntactic.parser.ast;

import org.adjective.syntactic.parser.node.BaseNode;
import org.adjective.syntactic.parser.node.HasAnnotations;
import org.adjective.syntactic.parser.node.HasModifiers;
import org.adjective.syntactic.parser.util.ModifierSet;

public class ASTResource extends BaseNode implements HasModifiers, HasAnnotations
{
    public ASTResource(int id)
    {
        super(id);
    }

    public <R, T> R jjtAccept(JavaParserVisitor<R, T> visitor, T data)
    {
        return visitor.visit(this, data);
    }

    public ASTVariableModifiers getModifiersNode()
    {
        return getFirstChild().as(ASTVariableModifiers.class);
    }

    @Override
    public Iterable<ASTAnnotation> getAnnotations()
    {
        return getModifiersNode().getAnnotations();
    }

    @Override
    public ModifierSet getModifiers()
    {
        return getModifiersNode().getModifiers();
    }

    public ASTReferenceType getDeclaredType()
    {
        return child(1).as(ASTReferenceType.class);
    }

    public ASTIdentifier getName()
    {
        return child(2).as(ASTIdentifier.class);
    }

    public ASTArraySuffixList getArraySuffix()
    {
        return child(3).as(ASTArraySuffixList.class);
    }

    public ASTExpression getAssignmentValue()
    {
        return getLastChild().as(ASTExpression.class);
    }
}
