/* ------------------------------------------------------------------------
 * Copyright 2012 Tim Vernum
 * ------------------------------------------------------------------------
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ------------------------------------------------------------------------
 */
package org.adjective.syntactic.parser.ast;

import org.adjective.syntactic.parser.node.BaseNode;
import org.adjective.syntactic.parser.jj.JavaParserConstants;
import org.adjective.syntactic.parser.node.ForLoopInit;

public class ASTTernaryForCondition extends BaseNode implements ASTForStatement.ForCondition
{
    public ASTTernaryForCondition(final int i)
    {
        super(i);
    }

    @Override
    public <R, T> R jjtAccept(final JavaParserVisitor<R, T> visitor, final T data)
    {
        return visitor.visit(this, data);
    }

    public ForLoopInit getInit()
    {
        if (hasInitStatement())
        {
            return getFirstChild().as(ForLoopInit.class);
        }
        else
        {
            return null;
        }
    }

    private boolean hasInitStatement()
    {
        return jjtGetFirstToken().kind != JavaParserConstants.SEMICOLON;
    }

    public ASTExpression getCondition()
    {
        return optionalChild(ASTExpression.class);
    }

    public ASTStatementExpressionList getUpdate()
    {
        if (hasUpdateStatement())
        {
            return optionalChild(ASTStatementExpressionList.class);
        }
        else
        {
            return null;
        }
    }

    private boolean hasUpdateStatement()
    {
        return jjtGetLastToken().kind != JavaParserConstants.SEMICOLON;
    }
}
