package org.adjective.syntactic.parser.ast;

import org.adjective.syntactic.parser.name.TypeParameter;
import org.adjective.syntactic.parser.node.BaseNode;
import org.adjective.syntactic.parser.node.TypeNode;
import org.adjective.syntactic.parser.type.JavaType;
import org.adjective.syntactic.parser.type.SimpleJavaType;

public class ASTTypeArgument extends BaseNode implements TypeParameter
{
    public ASTTypeArgument(int id)
    {
        super(id);
    }

    public ASTTypeArgument(final TypeParameter parameter)
    {
        this(JavaParserTreeConstants.JJTTYPEARGUMENT);
        final TypeNode typeNode = makeTypeNode(parameter.getType());
        final TypeParameter.Kind kind = parameter.getKind();
        if (kind == TypeParameter.Kind.EXACT)
        {
            appendChild(typeNode);
        }
        else
        {
            appendChild(new ASTWildcardBounds(kind, typeNode));
        }
    }

    public <R, T> R jjtAccept(JavaParserVisitor<R, T> visitor, T data)
    {
        return visitor.visit(this, data);
    }

    public boolean isWildcard()
    {
        return isEmpty() || getFirstChild().is(ASTWildcardBounds.class);
    }

    @Override
    public Kind getKind()
    {
        if (isEmpty())
        {
            return Kind.EXTENDS;
        }
        if (isWildcard())
        {
            return getFirstChild().as(ASTWildcardBounds.class).getKind();
        }
        else
        {
            return Kind.EXACT;
        }
    }

    @Override
    public JavaType getType()
    {
        if (isEmpty())
        {
            return new SimpleJavaType("Object");
        }
        if (isWildcard())
        {
            return getFirstChild().as(ASTWildcardBounds.class).getType();
        }
        else
        {
            return getFirstChild().as(TypeNode.class);
        }
    }
}
