package org.adjective.syntactic.parser.ast;

import org.adjective.syntactic.parser.node.BaseNode;
import org.adjective.syntactic.parser.node.HasAnnotations;
import org.adjective.syntactic.parser.node.HasModifiers;
import org.adjective.syntactic.parser.node.IsTypeDeclaration;
import org.adjective.syntactic.parser.type.DeclarationType;
import org.adjective.syntactic.parser.util.*;

import java.util.Collections;

public class ASTTypeDeclaration extends BaseNode implements HasAnnotations, HasModifiers
{
    public ASTTypeDeclaration(int id)
    {
        super(id);
    }

    public <R,T> R jjtAccept(JavaParserVisitor<R,T> visitor, T data)
    {
        return visitor.visit(this, data);
    }

    public ModifierSet getModifiers()
    {
        if (super.isEmpty())
        {
            return new ModifierSet();
        }
        else
        {
            return child(ASTModifiers.class).getModifiers();
        }
    }

    public DeclarationType getType()
    {
        if (super.isEmpty())
        {
            return DeclarationType.NOTHING;
        }
        else
        {
            return getDeclarationBody().getDeclarationType();
        }
    }

    public IsTypeDeclaration getDeclarationBody()
    {
        return child(1).as(IsTypeDeclaration.class);
    }

    @Override
    public Iterable<ASTAnnotation> getAnnotations()
    {
        if (super.isEmpty())
        {
            return Collections.emptySet();
        }
        else
        {
            return child(ASTModifiers.class).getAnnotations();
        }
    }
}
