/* ------------------------------------------------------------------------
 * Copyright 2012 Tim Vernum
 * ------------------------------------------------------------------------
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ------------------------------------------------------------------------
 */
package org.adjective.syntactic.parser.ast;

import org.adjective.syntactic.parser.node.BaseNode;
import org.adjective.syntactic.parser.jj.JavaParserConstants;
import org.adjective.syntactic.parser.jj.Token;

public class ASTUnaryOperator extends BaseNode
{
    public enum UnaryOperator
    {
        POSITIVE, NEGATIVE, BINARY_NOT, LOGICAL_NOT, INCREMENT, DECREMENT
    }

    public ASTUnaryOperator(final int i)
    {
        super(i);
    }

    @Override
    public <R, T> R jjtAccept(final JavaParserVisitor<R, T> visitor, final T data)
    {
        return visitor.visit(this, data);
    }

    public UnaryOperator getOperator()
    {
        final Token token = jjtGetFirstToken();
        switch (token.kind)
        {
            case JavaParserConstants.PLUS:
                return UnaryOperator.POSITIVE;
            case JavaParserConstants.MINUS:
                return UnaryOperator.NEGATIVE;
            case JavaParserConstants.TILDE:
                return UnaryOperator.BINARY_NOT;
            case JavaParserConstants.BANG:
                return UnaryOperator.LOGICAL_NOT;
            case JavaParserConstants.INCR:
                return UnaryOperator.INCREMENT;
            case JavaParserConstants.DECR:
                return UnaryOperator.DECREMENT;
        }
        throw new IllegalStateException("PostfixOperator token " + token.kind
                                                + " (" + JavaParserConstants.tokenImage[token.kind] + " ) "
                                                + token.image + " not understood");
    }
}
