/* ------------------------------------------------------------------------
 * Copyright 2012 Tim Vernum
 * ------------------------------------------------------------------------
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ------------------------------------------------------------------------
 */
package org.adjective.syntactic.parser.ast;

import org.adjective.syntactic.parser.node.BaseNode;
import org.adjective.syntactic.parser.node.HasAnnotations;
import org.adjective.syntactic.parser.node.HasModifiers;
import org.adjective.syntactic.parser.util.ModifierSet;

public class ASTVariableModifiers extends BaseNode implements HasAnnotations, HasModifiers
{
    private ModifierSet _modifiers = null;

    public ASTVariableModifiers(int i)
    {
        super(i);
    }

    public ASTVariableModifiers()
    {
        this(JavaParserTreeConstants.JJTVARIABLEMODIFIERS);
        setFinal(false);
    }

    @Override
    public <R,T> R jjtAccept(JavaParserVisitor<R,T> visitor, T data)
    {
        return visitor.visit(this, data);
    }

    @Override
    public Iterable<ASTAnnotation> getAnnotations()
    {
        return super.allChildren(ASTAnnotation.class);
    }

    public void setFinal(boolean isFinal)
    {
        if (_modifiers == null)
        {
            _modifiers = new ModifierSet();
            if (isFinal)
            {
                _modifiers.set(ModifierSet.Modifier.FINAL);
            }
        }
        else
        {
            throw new IllegalStateException("Modifiers for " + this + " are already set to " + _modifiers.toString());
        }
    }

    public ModifierSet getModifiers()
    {
        return _modifiers;
    }
}
