package org.adjective.syntactic.parser.ast;

import org.adjective.syntactic.parser.node.BaseNode;
import org.adjective.syntactic.parser.jj.JavaParserConstants;
import org.adjective.syntactic.parser.type.JavaType;
import org.adjective.syntactic.parser.node.TypeNode;
import org.adjective.syntactic.parser.name.TypeParameter;

public class ASTWildcardBounds extends BaseNode implements TypeParameter
{
    public ASTWildcardBounds(int id)
    {
        super(id);
    }

    public ASTWildcardBounds(final TypeParameter.Kind kind, final TypeNode typeNode)
    {
        this(JavaParserTreeConstants.JJTWILDCARDBOUNDS);
        if (kind == TypeParameter.Kind.EXTENDS)
        {
            jjtSetFirstToken(makeToken(JavaParserConstants.EXTENDS, "extends"));
        }
        else
        {
            jjtSetFirstToken(makeToken(JavaParserConstants.SUPER, "super"));
        }
        assert typeNode instanceof ASTReferenceType;
        setChildren((ASTReferenceType) typeNode);
    }

    public <R, T> R jjtAccept(JavaParserVisitor<R, T> visitor, T data)
    {
        return visitor.visit(this, data);
    }

    public boolean isExtendsBound()
    {
        return jjtGetFirstToken().kind == JavaParserConstants.EXTENDS;
    }

    public boolean isSuperBound()
    {
        return jjtGetFirstToken().kind == JavaParserConstants.SUPER;
    }

    public ASTReferenceType getBound()
    {
        return getFirstChild().as(ASTReferenceType.class);
    }

    @Override
    public Kind getKind()
    {
        return isExtendsBound() ? Kind.EXTENDS : Kind.SUPER;
    }

    @Override
    public JavaType getType()
    {
        return getBound();
    }
}
