package org.adjective.syntactic.parser.ast;

import org.adjective.syntactic.parser.jj.Token;
import org.adjective.syntactic.parser.node.HasTokens;

public class SimpleNode implements Node, HasTokens
{
    private Node parent;
    private Node[] children;
    private int id;
    private Object value;
    private Token firstToken;
    private Token lastToken;

    protected SimpleNode(int i)
    {
        id = i;
    }

    public void jjtOpen()
    {
    }

    public void jjtClose()
    {
    }

    public void jjtSetParent(Node n)
    {
        parent = n;
    }

    public Node jjtGetParent()
    {
        return parent;
    }

    public void jjtAddChild(Node n, int i)
    {
        if (n == null)
        {
            throw new IllegalArgumentException("Cannot add null child");
        }
        if (children == null)
        {
            children = new Node[i + 1];
        }
        else if (i >= children.length)
        {
            Node c[] = new Node[i + 1];
            System.arraycopy(children, 0, c, 0, children.length);
            children = c;
        }
        children[i] = n;
    }

    public Node jjtGetChild(int i)
    {
        return children[i];
    }

    public int jjtGetNumChildren()
    {
        return (children == null) ? 0 : children.length;
    }

    public void jjtSetValue(Object value)
    {
        this.value = value;
    }

    public Object jjtGetValue()
    {
        return value;
    }

    @Override
    public Token jjtGetFirstToken()
    {
        return firstToken;
    }

    public void jjtSetFirstToken(Token token)
    {
        this.firstToken = token;
    }

    @Override
    public Token jjtGetLastToken()
    {
        return lastToken;
    }

    public void jjtSetLastToken(Token token)
    {
        this.lastToken = token;
    }

    @Override
    public <R, T> R jjtAccept(JavaParserVisitor<R, T> visitor, T data)
    {
        throw new UnsupportedOperationException("AST Node " + getClass().getSimpleName() + " does not implement jjAccept");
    }

    public <T> void childrenAccept(JavaParserVisitor<?, T> visitor, T data)
    {
        if (children != null)
        {
            for (int i = 0; i < children.length; ++i)
            {
                children[i].jjtAccept(visitor, data);
            }
        }
    }

    public String toString()
    {
        return JavaParserTreeConstants.jjtNodeName[id];
    }

    public String toString(String prefix)
    {
        return prefix + toString();
    }

    public void dump(String prefix)
    {
        System.out.println(toString(prefix));
        if (children != null)
        {
            for (int i = 0; i < children.length; ++i)
            {
                SimpleNode n = (SimpleNode) children[i];
                if (n != null)
                {
                    n.dump(prefix + " ");
                }
            }
        }
    }

    public void jjtClearChildren()
    {
        this.children = new Node[0];
    }
}
