/* ------------------------------------------------------------------------
 * Copyright 2013 Tim Vernum
 * ------------------------------------------------------------------------
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ------------------------------------------------------------------------
 */
package org.adjective.syntactic.parser.node;

import org.adjective.syntactic.parser.ast.Node;

import java.util.Iterator;

class TypeCheckNodeList<T extends Node> extends AbstractNodeList<T>
{
    private final BaseNode _parent;
    private final Class<T> _type;
    private final int _start;
    private final int _end;
    private final boolean _skipNonMatching;

    public TypeCheckNodeList(final BaseNode parent, final Class<T> type, final boolean skipNonMatching)
    {
        this(parent, type, 0, -1, skipNonMatching);
    }

    public TypeCheckNodeList(final BaseNode parent, final Class<T> type, final int start, final int end, final boolean skipNonMatching)
    {
        _parent = parent;
        _type = type;
        _start = start;
        _end = end;
        _skipNonMatching = skipNonMatching;
    }

    @Override
    public Iterator<T> iterator()
    {
        return new TypeFilterNodeIterator<T>(_parent, _type, _start, getEnd(), _skipNonMatching);
    }

    private int getEnd()
    {
        if (_end == -1)
        {
            return _parent.jjtGetNumChildren();
        }
        else
        {
            return Math.min(_end, _parent.jjtGetNumChildren());
        }
    }

    @Override
    public int size()
    {
        if (_skipNonMatching)
        {
            return super.size();
        }
        else
        {
            // Shortcut : avoid iterating if we don't need to filter
            return getEnd() - _start;
        }
    }

    @Override
    public String toString()
    {
        if (_start == 0 && _end == -1)
        {
            if (_skipNonMatching)
            {
                return "(" + _parent + ").findChildren(" + _type + ")";
            }
            return "(" + _parent + ").allChildren(" + _type + ")";
        }
        return "(" + _parent + ").children(" + _type + "," + _start + "," + _end + "," + _skipNonMatching + ")";
    }
}
