/* ------------------------------------------------------------------------
 * Copyright 2013 Tim Vernum
 * ------------------------------------------------------------------------
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ------------------------------------------------------------------------
 */
package org.adjective.syntactic.parser.node;

import org.adjective.syntactic.parser.ast.Node;

import java.util.Iterator;
import java.util.NoSuchElementException;

class TypeFilterNodeIterator<T extends Node> implements Iterator<T>
{
    private final Class<T> _type;
    private final boolean _skipNonMatching;
    private final int _stop;
    private int _index;
    private BaseNode _parent;

    public TypeFilterNodeIterator(final BaseNode parent, Class<T> type, boolean skipNonMatching)
    {
        this(parent, type, 0, parent.jjtGetNumChildren(), skipNonMatching);
    }

    public TypeFilterNodeIterator(final BaseNode parent, Class<T> type, int start, final int end, boolean skipNonMatching)
    {
        _parent = parent;
        _type = type;
        _skipNonMatching = skipNonMatching;
        _index = start - 1;
        _stop = Math.min(end, _parent.jjtGetNumChildren());
        advance();
    }

    private void advance()
    {
        for (_index++; hasNext(); _index++)
        {
            Node node = _parent.jjtGetChild(_index);
            if (_type.isInstance(node) || !_skipNonMatching)
            {
                return;
            }
        }
    }

    @Override
    public boolean hasNext()
    {
        return _index < _stop;
    }

    @Override
    public T next()
    {
        if (!hasNext())
        {
            throw new NoSuchElementException();
        }
        Node node = _parent.jjtGetChild(_index);
        if (_type.isInstance(node))
        {
            advance();
            return _type.cast(node);
        }
        else
        {
            throw new IllegalStateException("Node " + node + " is not of type " + _type.getSimpleName());
        }
    }

    @Override
    public void remove()
    {
        throw new UnsupportedOperationException("remove() - not implemented");
    }
}
