/* ------------------------------------------------------------------------
 * Copyright 2012 Tim Vernum
 * ------------------------------------------------------------------------
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ------------------------------------------------------------------------
 */
package org.adjective.syntactic.parser.type;

import org.adjective.syntactic.parser.name.ParameterizedName;

public class ArrayType implements JavaType
{
    private final JavaType _base;
    private final int _arrayDepth;

    public ArrayType(JavaType base, int arrayDepth)
    {
        _base = base;
        _arrayDepth = arrayDepth;
    }

    @Override
    public BaseType getBaseType()
    {
        return _base.getBaseType();
    }

    @Override
    public String getTypeName()
    {
        return _base.getTypeName();
    }

    @Override
    public ParameterizedName[] getParameterizedTypeName()
    {
        return _base.getParameterizedTypeName();
    }

    @Override
    public int getArrayDepth()
    {
        return _base.getArrayDepth() + _arrayDepth;
    }
}
