/* ------------------------------------------------------------------------
 * Copyright 2013 Tim Vernum
 * ------------------------------------------------------------------------
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ------------------------------------------------------------------------
 */
package org.adjective.syntactic.util;

import org.adjective.syntactic.parser.name.ParameterizedName;
import org.adjective.syntactic.parser.name.SimpleParameterizedName;
import org.adjective.syntactic.parser.name.SimpleTypeParameter;
import org.adjective.syntactic.parser.name.TypeParameter;
import org.adjective.syntactic.parser.type.JavaType;
import org.adjective.syntactic.parser.type.SimpleJavaType;

public class JavaTypeUtil
{
    public static boolean isMatchingType(final JavaType type, final Class<?> cls)
    {
        return cls.getName().equals(type.getTypeName());
    }

    public static JavaType withoutWildcard(final JavaType forType)
    {
        if (!hasWildcard(forType))
        {
            return forType;
        }
        final ParameterizedName[] inputName = forType.getParameterizedTypeName();
        final ParameterizedName[] outputName = new ParameterizedName[inputName.length];
        for (int i = 0; i < inputName.length; i++)
        {
            ParameterizedName parameterizedName = inputName[i];
            final TypeParameter[] inputParameters = parameterizedName.getParameters();
            if (inputParameters.length == 0)
            {
                outputName[i] = parameterizedName;
                continue;
            }
            final TypeParameter[] outputParameters = new TypeParameter[inputParameters.length];
            for (int j = 0; j < outputParameters.length; j++)
            {
                outputParameters[j] = new SimpleTypeParameter(TypeParameter.Kind.EXACT, inputParameters[j].getType());
            }
            outputName[i] = new SimpleParameterizedName(inputName[i].getName(), outputParameters);
        }
        return new SimpleJavaType(outputName, forType.getArrayDepth());
    }

    private static boolean hasWildcard(final JavaType forType)
    {
        final TypeParameter[] parameters = ArrayUtil.last(forType.getParameterizedTypeName()).getParameters();
        for (final TypeParameter parameter : parameters)
        {
            if (parameter.getKind() != TypeParameter.Kind.EXACT)
            {
                return true;
            }
        }
        return false;
    }
}
