/* ------------------------------------------------------------------------
 * Copyright 2013 Tim Vernum
 * ------------------------------------------------------------------------
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ------------------------------------------------------------------------
 */

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

public class GenericLambda
{
    public interface Transformer<FROM, TO>
    {
        public TO transform(FROM from);
    }

    public static <IN, OUT> List<OUT> transform(Iterable<? extends IN> list, Transformer<? super IN, ? extends OUT> transformer)
    {
        List<OUT> result = new ArrayList<OUT>();
        for (IN in : list)
        {
            result.add(transformer.transform(in));
        }
        return result;
    }

    public static void main(String[] args)
    {
        final Transformer<? super Long, CharSequence> transformer = (Number n) -> n.toString();
        transform(Collections.singletonList(99L), transformer);
    }

}
