/* ------------------------------------------------------------------------
 * Copyright 2013 Tim Vernum
 * ------------------------------------------------------------------------
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ------------------------------------------------------------------------
 */
package org.adjective.syntactic.convert.j8to7.functional;

import org.adjective.syntactic.convert.j8to7.Convert8To7Visitor;
import org.adjective.syntactic.parser.ast.ASTCompilationUnit;
import org.adjective.syntactic.parser.jj.JavaParser;
import org.adjective.syntactic.parser.util.PrintSourceVisitor;
import org.adjective.syntactic.util.IOUtil;
import org.junit.*;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;

public class ConvertFunctionalTest
{
    @org.junit.Test
    public void testConversions() throws Exception
    {
        File root = new File("convert");
        if (!root.exists())
        {
            root = new File(".");
        }
        final File directory = new File(root, "source/resource/test/functional/");
        if (!directory.exists())
        {
            Assert.fail("Directory '" + directory.getAbsolutePath() + "' does not exist");
        }
        final File[] files = directory.listFiles();
        if (files == null)
        {
            return;
        }
        for (File file : files)
        {
            if (isJavaSource(file))
            {
                File expected = new File(file.getAbsolutePath() + ".convert");
                convertAndVerify(file, expected);
            }
        }
    }

    private boolean isJavaSource(final File file)
    {
        return file.isFile() && file.canRead() && file.getName().endsWith(".java");
    }

    private void convertAndVerify(final File input, final File expected) throws IOException
    {
        final ASTCompilationUnit ast = JavaParser.process(input);
        if (ast == null)
        {
            Assert.fail("Failed to parse " + input);
        }
        if (!expected.exists())
        {
            System.err.println("Output '" + expected.getAbsolutePath() + "' for '" + input + "' does not exist");
            return;
        }
        System.err.println("Running test: " + input);
        new Convert8To7Visitor().convert(ast);
        final ByteArrayOutputStream output = new ByteArrayOutputStream();
        new PrintSourceVisitor().print(ast, output);
        final byte[] read = IOUtil.read(new FileInputStream(expected));
        Assert.assertEquals(new String(read), new String(output.toByteArray()));
    }
}
